# Copyright (C) 2007 by Intevation GmbH
# Authors:
# Bernhard Herzog <bh@intevation.de>
#
# This program is free software under the GPL (>=v2)
# Read the file GPL.txt coming with the software for details.

"""Database backend of OSAAS"""

try:
    import cx_Oracle as dbapi
except ImportError:
    class dbapi(object):

        def connect(*args):
            raise RuntimeError("Trying to call DBAPI methods"
                               " without a DBAPI module")
        connect = staticmethod(connect)

class DBField(object):

    def __init__(self, parameter, column, column_type):
        self.parameter = parameter
        self.column = column
        self.column_type = column_type


class Insertion(object):

    def __init__(self, table, fields):
        self.table = table
        self.fields = fields


class DBBackend(object):

    def __init__(self, connection_parameters, rules, logger=None):
        self.connection = None
        self.connection_parameters = connection_parameters
        self.rules = rules
        self.logger = logger

    def log_debug(self, message, *args):
        if self.logger is not None:
            self.logger.debug(message, *args)

    def log_error(self, message, *args):
        if self.logger is not None:
            self.logger.error(message, *args)

    def ensure_connection(self):
        if self.connection is None:
            self.log_debug("Connecting to database")
            self.connection = self.connect(*self.connection_parameters)

    def connect(self, *args):
        return dbapi.connect(*args)

    def handle_record(self, record):
        self.ensure_connection()
        rule = self.rules[0]
        names = ", ".join([f.column for f in rule.fields])
        statement = ("INSERT INTO %s (ID, %s) VALUES (%s.nextval, %s)"
                     % (rule.table, names, rule.table + "_seq",
                        ", ".join([":%s" % f.column for f in rule.fields])))
        values = {}
        for f in rule.fields:
            values[f.column] = getattr(record, f.parameter)
        cursor = self.connection.cursor()
        try:
            cursor.execute(statement, values)
        except:
            self.log_error("could not write record %s", record.raw_record)
            self.connection.rollback()
            raise
        else:
            self.connection.commit()

    def create_tables(self):
        """Creates the tables and sequences for the rules"""
        self.ensure_connection()
        cursor = self.connection.cursor()
        try:
            for rule in self.rules:
                cursor.execute("CREATE SEQUENCE %s" % (rule.table + "_seq"))

                lines = []
                lines.append("CREATE TABLE %s" % rule.table)
                lines.append("(")
                fieldlines = ["    ID INT PRIMARY KEY"]
                for field in rule.fields:
                    fieldlines.append("    %s %s"
                                      % (field.column, field.column_type))
                lines.append(",\n".join(fieldlines))
                lines.append(")")
                statement = "\n".join(lines)
                cursor.execute(statement)
        except:
            self.connection.rollback()
            raise
        else:
            self.connection.commit()
