#! /bin/env python
# Copyright (C) 2007 by Intevation GmbH
# Authors:
# Bernhard Herzog <bh@intevation.de>
#
# This program is free software under the GPL (>=v2)
# Read the file GPL.txt coming with the software for details.

"""Functions to run a httpserver as a standalone program"""

import sys
import os
import optparse
import logging
import signal

import httpserver


class SIGTERMException(Exception):
    pass


class OptionStack(object):

    def __init__(self, option_list, default_opts, file_opts, commandline_opts):
        self.option_list = option_list
        self.__opts = (commandline_opts, file_opts, default_opts)

    def __getattr__(self, attr):
        for opts in self.__opts:
            if hasattr(opts, attr):
                return getattr(opts, attr)
        raise AttributeError(attr)

    def set_file_option(self, name, value):
        setattr(self.__opts[1], name, value)

class ProgramWithOptions(object):

    optparse_option_class = None

    def create_option_parser(self):
        kw = {}
        if self.optparse_option_class is not None:
            kw["option_class"] = self.optparse_option_class
        parser = optparse.OptionParser(**kw)
        parser.add_option("--config-file")
        return parser

    def read_config_file(self, opts):
        """Reads options from a configuration file

        The parameter opts is an OptionStack instance holding defaults
        and values read from the command line.  Derived classes wishing
        to read config files should override this method and set options
        in opts with its set_file_option method.

        How the configuration file(s) are located and how settings are
        read is up to derived classes.  However, the base class already
        provides the config_file option as a standard way to provide the
        name of a configuration file on the command line.
        """

    def parse_options(self, args):
        parser = self.create_option_parser()

        default_opts = parser.get_default_values()
        configfile_opts = optparse.Values()
        commandline_opts, rest = parser.parse_args(args, optparse.Values())
        opts = OptionStack(parser.option_list, default_opts, configfile_opts,
                           commandline_opts)
        self.read_config_file(opts)
        return opts, rest


class ServerProgram(ProgramWithOptions):

    """Main program of the server"""

    default_port = 8989

    def __init__(self):
        self.server = None

    def create_option_parser(self, **kw):
        parser = ProgramWithOptions.create_option_parser(self, **kw)
        parser.set_defaults(port=self.default_port)
        parser.add_option("--port", type="int")
        parser.add_option("--access-log")
        parser.add_option("--error-log")
        return parser

    def setup_signals(self):
        """Set handle_term_signal as signal handler for SIGTERM"""
        signal.signal(signal.SIGTERM, self.handle_term_signal)

    def handle_term_signal(self, signum, frame):
        """Handler for the SIGTERM signal.  Throws SIGTERMException"""
        raise SIGTERMException

    def setup_logging(self, opts):
        """Sets up the default loggers of the httpserver based on opts.
        The function sets up the httpserver.access and httpserver.error
        loggers.  If opts has attributes access_log or error_log, they
        are assumed to be the names of the files the corresponding
        logger should write to.  If one of the attributes does not exist
        or is None or an empty string, the corresponding logger writes
        to sys.stderr.
        """
        for logtype, format in [("access", "%(message)s"),
                                ("error",
                            "%(asctime)s %(levelname)s %(name)s %(message)s")]:
            logger = logging.getLogger("httpserver." + logtype)
            logger.setLevel(logging.DEBUG)

            filename = getattr(opts, logtype + "_log")
            if filename:
                stream = open(filename, "a")
            else:
                stream = sys.stderr
            hdlr = logging.StreamHandler(stream)
            hdlr.setFormatter(logging.Formatter(format))
            logger.addHandler(hdlr)

    def instantiate_server(self, server_class, server_address, opts, **kw):
        return server_class(server_address, **kw)

    def main(self, server_class=httpserver.HTTPServer,
             default_port=8989, bind_host="127.0.0.1", **kw):
        opts, rest = self.parse_options(sys.argv[1:])

        self.setup_signals()
        self.setup_logging(opts)

        logger = logging.getLogger("httpserver.error")

        logger.info("server starting up")
        server_address = (bind_host, opts.port)
        self.server = self.instantiate_server(server_class, server_address,
                                              opts, **kw)

        print "Serving HTTP on port", self.server.server_port, "..."
        logger.info("serving HTTP on port %d", self.server.server_port)

        try:
            self.server.serve_forever()
        except (KeyboardInterrupt, SIGTERMException), err:
            if isinstance(err, KeyboardInterrupt):
                logger.info("keyboard interrupt; stopping server")
            else:
                logger.info("received SIGTERM; stopping server")
            self.server.server_close()

        logmessage = "server stopped"
        print logmessage
        logger.info(logmessage)

def main(*args, **kw):
    program = ServerProgram()
    program.main(*args, **kw)
