# Copyright (C) 2007 by Intevation GmbH
# Authors:
# Bernhard Herzog <bh@intevation.de>
#
# This program is free software under the GPL (>=v2)
# Read the file GPL.txt coming with the software for details.

import unittest

from osaas.dbbackend import DBBackend, Insertion, DBField
from osaas.formparser import Record


class MockConnection(object):

    def __init__(self, testcase):
        self.testcase = testcase

    def cursor(self):
        return MockCursor(self.testcase)

    def commit(self):
        self.testcase.method_called("Connection.commit")

    def rollback(self):
        self.testcase.method_called("Connection.rollback")


class MockCursor(object):

    def __init__(self, testcase):
        self.testcase = testcase

    def execute(self, statement, *args, **kw):
        self.testcase.failUnless(isinstance(statement, str))
        self.testcase.method_called("Cursor.execute", statement,
                                    *args, **kw)

class TestBackend(DBBackend):

    def __init__(self, testcase, *args):
        self.testcase = testcase
        DBBackend.__init__(self, *args)

    def connect(self, *args):
        self.testcase.assertEquals(args, self.connection_parameters)
        return MockConnection(self.testcase)

class MockLogger(object):

    def __init__(self):
        self.log = []

    def __getattr__(self, name):
        def logmethod(*args):
            self.log.append((name, args))
        return logmethod


class TestDBBackend(unittest.TestCase):

    def setUp(self):
        self.method_log = []
        self.logger = MockLogger()

    def method_called(self, name, *args, **kw):
        self.check_method_called(name, *args, **kw)
        entry = [name]
        if args:
            entry.append(args)
        if kw:
            entry.append(args)
        self.method_log.append(tuple(entry))

    def check_method_called(self, name, *args, **kw):
        """Called by method_called so that tests can simulate errors"""
        pass

    def test_create_tables(self):
        backend = TestBackend(self,
                              ("someuser", "aPassword", "mydns"),
                              [Insertion("mytable",
                                         [DBField("REQUEST", "req",
                                                  "character varying(11)")])],
                              self.logger)
        backend.create_tables()

        self.assertEquals(self.method_log,
                          [('Cursor.execute', ('CREATE SEQUENCE mytable_seq',)),
                           ("Cursor.execute", ("CREATE TABLE mytable\n"
                                               "(\n"
                                               "    ID INT PRIMARY KEY,\n"
                                               "    req character varying(11)\n"
                                               ")",)),
                           ("Connection.commit",)])
        self.assertEquals(self.logger.log,
                          [('debug', ('Connecting to database',))])

    def test_create_tables_error(self):
        def check_method_called(name, *args, **kw):
            if name == "Cursor.execute" and args[0].startswith("CREATE TABLE"):
                raise RuntimeError("Simulated database error")
        self.check_method_called = check_method_called

        backend = TestBackend(self,
                              ("someuser", "aPassword", "mynds"),
                              [Insertion("mytable",
                                         [DBField("REQUEST", "req",
                                                  "character varying(11)")])])
        try:
            backend.create_tables()
        except RuntimeError:
            pass

        self.assertEquals(self.method_log,
                          [('Cursor.execute', ('CREATE SEQUENCE mytable_seq',)),
                           ("Connection.rollback",)])

    def test_handle_record(self):
        backend = TestBackend(self,
                              ("someuser", "aPassword", "mydns"),
                              [Insertion("mytable",
                                         [DBField("REQUEST", "req",
                                                  "character varying(11)")])],
                              self.logger)
        record = Record()
        record.REQUEST = "SERVICE=WMS&REQUEST=GetMap"
        backend.handle_record(record)

        self.assertEquals(self.method_log,
                          [("Cursor.execute",
                            ("INSERT INTO mytable (ID, req)"
                             " VALUES (mytable_seq.nextval, :req)",
                             {"req": "SERVICE=WMS&REQUEST=GetMap"})),
                           ("Connection.commit",)])
        self.assertEquals(self.logger.log,
                          [('debug', ('Connecting to database',))])

    def test_handle_record_error(self):
        def check_method_called(name, *args, **kw):
            if name == "Cursor.execute" and args[0].startswith("INSERT"):
                raise RuntimeError("Simulated database error")
        self.check_method_called = check_method_called
        backend = TestBackend(self,
                              ("someuser", "aPassword", "mydns"),
                              [Insertion("mytable",
                                         [DBField("REQUEST", "req",
                                                  "character varying(11)")])],
                              self.logger)
        record = Record()
        record.REQUEST = "SERVICE=WMS&REQUEST=GetMap"
        record.raw_record = "REQUEST=SERVICE=WMS%26REQUEST=GetMap"
        try:
            backend.handle_record(record)
        except RuntimeError:
            pass

        self.assertEquals(self.method_log,
                          [("Connection.rollback",)])
        self.assertEquals(self.logger.log,
                          [('debug', ('Connecting to database',)),
                           ('error', ('could not write record %s',
                                      'REQUEST=SERVICE=WMS%26REQUEST=GetMap'))])
