# Copyright (C) 2007 by Intevation GmbH
# Authors:
# Bernhard Herzog <bh@intevation.de>
#
# This program is free software under the GPL (>=v2)
# Read the file GPL.txt coming with the software for details.

"""Tests for ThreadPool"""

from __future__ import nested_scopes

import logging
import unittest

from serversupport import LoggerMixin

from osaas.http.threadpool import ThreadPool

class TestThreadPool(unittest.TestCase, LoggerMixin):

    def setUp(self):
        self.log_stream = self.new_logger("threadpool",
                               format="%(levelname)s %(name)s %(message)s")
        self.logger = logging.getLogger("threadpool")

    def test_threadpool_start_stop_logging(self):
        def handler(item):
            pass
        pool = ThreadPool(2, handler, logger=self.logger)
        pool.start()
        pool.stop()
        self.assertEquals(self.log_stream.getvalue(),
                          "INFO threadpool Starting 2 threads\n"
                          "INFO threadpool Stopping threads\n")

    def test_threadpool_error_handling(self):
        # If the handler function of a thread pool raises an exception,
        # the thread must not stop.  It must continue to take items out
        # of the thread pool's queue and process them.  To test this we
        # use a handler function that records whether it has been called
        # and then raises an exception.  We put more items into the
        # thread pool's queue than there are threads in the queue.  In
        # the end, after the thread pool has been stopped, the handler
        # function must have been called once for each item put into the
        # queue.
        record = []
        def error(item):
            record.append(item)
            raise RuntimeError

        pool = ThreadPool(2, error, logger=self.logger)
        pool.start()

        for i in range(3):
            pool.put(i)

        pool.stop()
        self.assertEquals(sorted(record), range(3))

        # rudimentary check that the errors were logged.
        self.log_stream.seek(0)
        self.assertEquals([line.strip() for line in self.log_stream.readlines()
                           if line.startswith("ERROR")],
                          ["ERROR threadpool Exception raised by"
                           " threadpool handler function"] * 3)
